/*
 * Copyright (c) 2003 Patrick van Staveren <pvanstav@cs.wmich.edu>
 * Copyright (c) 2005 Mario Streiber <mario.streiber@gmx.de>
 * Copyright (c) 2005-2009 Kemal Ilgar Eroglu <ilgar_eroglu@yahoo.com> 
 * 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef AUDACIOUS_OLD
#include <glib.h>
#include <glib-object.h>
#include <audacious/audctrl.h>
#include <audacious/dbus.h>
#else
#include <audacious/beepctrl.h>
#endif

#include "libcommon.h"

gboolean player_init (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	DBusGProxy *session=NULL;
	DBusGConnection *connection = NULL;
	g_type_init();
	connection = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
	if (!connection) {
		return FALSE;
	}
	session = dbus_g_proxy_new_for_name(connection, AUDACIOUS_DBUS_SERVICE,
		AUDACIOUS_DBUS_PATH, AUDACIOUS_DBUS_INTERFACE);
	if (!session) {
		return FALSE;
	}	
	
	pd->dbusproxy = (gpointer) session;
#endif	
	return TRUE;
}	

gboolean player_deinit (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	if (pd->dbusproxy)
		g_object_unref((DBusGProxy *) pd->dbusproxy);
#endif
	
	return TRUE;
	
}	


void player_save_special_options (gpointer cdata) {
	
}

void player_read_special_options (gpointer cdata) {
	
}

void player_populate_special_options (gpointer cdata) {
	
}

gboolean player_update_special_options (gpointer cdata) {

	return FALSE;
}

guint player_get_menu_options (gpointer cdata) {

  return (ITEM_ADD | ITEM_OPEN | ITEM_CLEAR | ITEM_REPEAT | ITEM_SHUFFLE | ITEM_VIEW_MAIN | ITEM_VIEW_PL |ITEM_VIEW_EQ | ITEM_PREFS | ITEM_POWER );
  
}

void player_playlist (gpointer cdata, gchar ** list, gint num, gboolean enqueue) {
	

}


void  player_play (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD	
	audacious_remote_activate((DBusGProxy *) pd->dbusproxy);
	audacious_remote_play((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_play(pd->xmms_session);
#endif
}	


void player_pause (gpointer cdata) {
	
	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	audacious_remote_pause((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_pause(pd->xmms_session);
#endif
}


void player_stop (gpointer cdata) {
	
	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	audacious_remote_stop((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_stop(pd->xmms_session);
#endif
}

gboolean player_is_playing (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return audacious_remote_is_playing((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_is_playing(pd->xmms_session);
#endif
}


gboolean player_is_paused (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	return audacious_remote_is_paused ((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_is_paused (pd->xmms_session);
#endif
}

gint player_get_playlist_pos (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD
	return audacious_remote_get_playlist_pos((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_get_playlist_pos(pd->xmms_session);
#endif
}


void player_set_playlist_pos (gpointer cdata, gint pos) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	audacious_remote_set_playlist_pos ((DBusGProxy *) pd->dbusproxy, (guint) pos);
#else
	audacious_remote_set_playlist_pos (pd->xmms_session, (guint) pos);
#endif
}


gint player_get_playlist_length (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	audacious_remote_get_playlist_length((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_get_playlist_length(pd->xmms_session);
#endif
}


gint player_get_output_time (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD	
	return audacious_remote_get_output_time((DBusGProxy *) pd->dbusproxy)/1000;
#else
	return audacious_remote_get_output_time(pd->xmms_session)/1000;
#endif
}

void player_jump_to_time (gpointer cdata, gint pos) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD	
	audacious_remote_jump_to_time ((DBusGProxy *) pd->dbusproxy, (guint) pos *1000);
#else
	audacious_remote_jump_to_time (pd->xmms_session, (guint) pos *1000);
#endif
}


void player_get_volume (gpointer cdata, gint * vl, gint * vr) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	audacious_remote_get_volume ((DBusGProxy *) pd->dbusproxy, vl ,vr);
#else
	audacious_remote_get_volume (pd->xmms_session, vl ,vr);
#endif
}


void player_set_volume (gpointer cdata, gint vl, gint vr) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD	
	audacious_remote_set_volume (((DBusGProxy *) pd->dbusproxy), vl ,vr);
#else
	audacious_remote_set_volume (pd->xmms_session, vl ,vr);
#endif
}


gchar * player_get_playlist_file (gpointer cdata, gint pos) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	return g_strdup(audacious_remote_get_playlist_file((DBusGProxy *) pd->dbusproxy, (guint) pos));
#else
	return g_strdup(audacious_remote_get_playlist_file(pd->xmms_session, (guint) pos));
#endif
}


gchar * player_get_playlist_title (gpointer cdata, gint pos) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return g_strdup(audacious_remote_get_playlist_title((DBusGProxy *) pd->dbusproxy, (guint) pos));
#else
	return g_strdup(audacious_remote_get_playlist_title(pd->xmms_session, (guint) pos));
#endif
}


gint player_get_playlist_time (gpointer cdata, gint pos) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD	
	return audacious_remote_get_playlist_time((DBusGProxy *) pd->dbusproxy, (guint) pos)/1000;
#else
	return audacious_remote_get_playlist_time(pd->xmms_session, (guint) pos)/1000;
#endif
}

void player_show_prefs_box (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	audacious_remote_show_prefs_box((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_show_prefs_box(pd->xmms_session);
#endif	
}

void player_eject (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	audacious_remote_eject((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_eject(pd->xmms_session);
#endif
}


void player_playlist_prev (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	audacious_remote_playlist_prev((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_playlist_prev(pd->xmms_session);
#endif
}


void player_playlist_next (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD	
	audacious_remote_playlist_next((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_playlist_next(pd->xmms_session);
#endif
}


gboolean player_is_running (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return audacious_remote_is_running((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_is_running(pd->xmms_session);
#endif
}


void  player_quit (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD	
	audacious_remote_quit((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_quit(pd->xmms_session);
#endif
}

void  player_play_pause (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD	
	audacious_remote_play_pause((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_play_pause(pd->xmms_session);
#endif	
}

gchar *  player_get_version (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return g_strdup(audacious_remote_get_version((DBusGProxy *) pd->dbusproxy));
#else
	return return g_strdup_printf("%d",audacious_remote_get_version( ((plugin_data *) cdata)->xmms_session));
#endif	
}


void  player_playlist_add (gpointer cdata, GList * list) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD
	audacious_remote_playlist_add ((DBusGProxy *) pd->dbusproxy,	list);
#else
	audacious_remote_playlist_add (pd->xmms_session,	list);
#endif		
}


void  player_playlist_delete (gpointer cdata, gint pos) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	audacious_remote_playlist_delete ((DBusGProxy *) pd->dbusproxy, pos);
#else
	audacious_remote_playlist_delete (pd->xmms_session, pos);
#endif		
}


void  player_main_win_toggle (gpointer cdata, gboolean show) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	audacious_remote_main_win_toggle ((DBusGProxy *) pd->dbusproxy, show);
#else
	audacious_remote_main_win_toggle (pd->xmms_session, show);
#endif	
}


void  player_pl_win_toggle (gpointer cdata, gboolean show) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD
	audacious_remote_pl_win_toggle ((DBusGProxy *) pd->dbusproxy, show);
#else
	audacious_remote_pl_win_toggle (pd->xmms_session, show);
#endif		
}


void  player_eq_win_toggle (gpointer cdata, gboolean show) {
	
	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	audacious_remote_eq_win_toggle ((DBusGProxy *) pd->dbusproxy, show);
#else
	audacious_remote_eq_win_toggle (pd->xmms_session, show);
#endif	
}


gboolean  player_is_main_win (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return audacious_remote_is_main_win ((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_is_main_win (pd->xmms_session);
#endif	

}


gboolean  player_is_pl_win (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return audacious_remote_is_pl_win ((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_is_pl_win (pd->xmms_session);
#endif	

}


gboolean  player_is_eq_win (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return audacious_remote_is_eq_win ((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_is_eq_win (pd->xmms_session);
#endif		
}


void  player_playlist_add_url_string (gpointer cdata, gchar * string) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	audacious_remote_playlist_add_url_string ((DBusGProxy *) pd->dbusproxy, string);
#else
	audacious_remote_playlist_add_url_string (pd->xmms_session, string);
#endif	
	
}

void  player_toggle_repeat (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	audacious_remote_toggle_repeat ((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_toggle_repeat (pd->xmms_session);
#endif	
}

void  player_toggle_shuffle (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	audacious_remote_toggle_shuffle ((DBusGProxy *) pd->dbusproxy);
#else
	audacious_remote_toggle_shuffle (pd->xmms_session);
#endif	
}


gboolean  player_is_repeat (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD
	return audacious_remote_is_repeat ((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_is_repeat (pd->xmms_session);
#endif	
}	


gboolean  player_is_shuffle (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return audacious_remote_is_shuffle((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_is_shuffle(pd->xmms_session);
#endif		
}

void  player_playlist_clear (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
		
#ifndef AUDACIOUS_OLD
	return audacious_remote_playlist_clear ((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_playlist_clear (pd->xmms_session);
#endif	
}
	
void  player_show_jtf_box (gpointer cdata) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	return audacious_remote_show_jtf_box ((DBusGProxy *) pd->dbusproxy);
#else
	return audacious_remote_show_jtf_box (pd->xmms_session);
#endif		
}

void player_toggle_aot (gpointer cdata, gboolean ontop) {

	plugin_data *pd = (plugin_data *)cdata;
	
#ifndef AUDACIOUS_OLD
	audacious_remote_toggle_aot ((DBusGProxy *) pd->dbusproxy,ontop);
#else
	audacious_remote_toggle_aot (pd->xmms_session,ontop);
#endif	
}
