/*
 * Copyright (c) 2003 Patrick van Staveren <pvanstav@cs.wmich.edu>
 * Copyright (c) 2005 Mario Streiber <mario.streiber@gmx.de>
 * Copyright (c) 2005-2009 Kemal Ilgar Eroglu <ilgar_eroglu@yahoo.com> 
 * 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib.h>
#include <gmodule.h>
#include <stdio.h>

#include "playerctrl.h"

/**
 * playerctrl.c
 * This is wrapper functions for XMMS API.
 * It enables dynamic loading of the player's library
 */

gchar * player_control_get_command(Players id) {
	gchar *cmdstr;
	switch (id) {
		case PLAYER_XMMS:
			cmdstr = CMD_XMMS;
			break;
		case PLAYER_BEEP:
			cmdstr = CMD_BEEP;
			break;
		case PLAYER_AUDACIOUS:
			cmdstr = CMD_AUDACIOUS;
			break;
		default:
			return NULL;
			break;
	}
	return cmdstr;
}

PlayerCtrl * player_control_get_instance(Players id) {
	gchar *libname;
	switch (id) {
		case PLAYER_XMMS:
			libname = "libplayercontrol_xmms";
			break;
		case PLAYER_BEEP:
			libname = "libplayercontrol_beep";
			break;
		case PLAYER_AUDACIOUS:
			libname = "libplayercontrol_audacious";
			break;
		case PLAYER_AUDACIOUS_OLD:
			libname = "libplayercontrol_audacious_old";
			break;
		case PLAYER_MPD:
			libname = "libplayercontrol_mpd";
			break;
		case PLAYER_AMAROK:
			libname = "libplayercontrol_amarok";
			break;
		case PLAYER_NONE:
			libname = "libplayercontrol_dummy";
			break;			
		default:
			libname = "libplayercontrol_dummy";
			break;
	}
	return player_control_load_library(libname);
		
}

void player_control_close(PlayerCtrl *instance) {
	if (instance->library)
		g_module_close(instance->library);
	g_free(instance);
}

gboolean player_control_has_library(gchar *libname) {
	GModule *library;
	gchar *filename;

	filename = g_module_build_path(NULL, libname);
	library = g_module_open(filename, G_MODULE_BIND_LAZY);
	g_free(filename);

	if (library) {
		g_module_close(library);
		return TRUE;
	} else {
		return FALSE;
	}
}

PlayerCtrl * player_control_load_library(gchar *libname) {
	PlayerCtrl *instance;
	gchar *filename;


	instance = g_new0(PlayerCtrl, 1);

	//xfce_err(MODULE_DIR);
	filename = g_module_build_path(MODULE_DIR, libname);
	//xfce_err(filename);
	instance->library = g_module_open(filename, G_MODULE_BIND_LAZY);
	//xfce_err(g_module_error());
	g_free(filename);

	if (! instance->library){
		xfce_err("Couldn't open module");
		return NULL;
	}
	/********************* start binding *********************/

	g_module_symbol(instance->library, "player_init",
			(gpointer) &instance->player_init);
	g_module_symbol(instance->library, "player_deinit",
			(gpointer) &instance->player_deinit);			
	g_module_symbol(instance->library, "player_save_special_options",
			(gpointer) &instance->player_save_special_options);
	g_module_symbol(instance->library, "player_read_special_options",
			(gpointer) &instance->player_read_special_options);
	g_module_symbol(instance->library, "player_populate_special_options",
			(gpointer) &instance->player_populate_special_options);			
	g_module_symbol(instance->library, "player_update_special_options",
			(gpointer) &instance->player_update_special_options);			
	g_module_symbol(instance->library, "player_get_menu_options",
			(gpointer) &instance->player_get_menu_options);		
	g_module_symbol(instance->library, "player_playlist",
			(gpointer) &instance->player_playlist);
	g_module_symbol(instance->library, "player_play",
			(gpointer) &instance->player_play);
	g_module_symbol(instance->library, "player_pause",
			(gpointer) &instance->player_pause);
	g_module_symbol(instance->library, "player_stop",
			(gpointer) &instance->player_stop);
	g_module_symbol(instance->library, "player_is_playing",
			(gpointer) &instance->player_is_playing);
	g_module_symbol(instance->library, "player_is_paused",
			(gpointer) &instance->player_is_paused);
	g_module_symbol(instance->library, "player_get_playlist_pos",
			(gpointer) &instance->player_get_playlist_pos);
	g_module_symbol(instance->library, "player_set_playlist_pos",
			(gpointer) &instance->player_set_playlist_pos);
	g_module_symbol(instance->library, "player_get_playlist_length",
			(gpointer) &instance->player_get_playlist_length);
	g_module_symbol(instance->library, "player_playlist_clear",
			(gpointer) &instance->player_playlist_clear);
	g_module_symbol(instance->library, "player_get_output_time",
			(gpointer) &instance->player_get_output_time);
	g_module_symbol(instance->library, "player_jump_to_time",
			(gpointer) &instance->player_jump_to_time);
	g_module_symbol(instance->library, "player_get_volume",
			(gpointer) &instance->player_get_volume);
	g_module_symbol(instance->library, "player_set_volume",
			(gpointer) &instance->player_set_volume);
	g_module_symbol(instance->library, "player_get_playlist_file",
			(gpointer) &instance->player_get_playlist_file);
	g_module_symbol(instance->library, "player_get_playlist_title",
			(gpointer) &instance->player_get_playlist_title);
	g_module_symbol(instance->library, "player_get_playlist_time",
			(gpointer) &instance->player_get_playlist_time);
	g_module_symbol(instance->library, "player_show_prefs_box",
			(gpointer) &instance->player_show_prefs_box);
	g_module_symbol(instance->library, "player_toggle_aot",
			(gpointer) &instance->player_toggle_aot);
	g_module_symbol(instance->library, "player_eject",
			(gpointer) &instance->player_eject);
	g_module_symbol(instance->library, "player_playlist_prev",
			(gpointer) &instance->player_playlist_prev);
	g_module_symbol(instance->library, "player_playlist_next",
			(gpointer) &instance->player_playlist_next);
	g_module_symbol(instance->library, "player_is_running",
			(gpointer) &instance->player_is_running);
	/* Added in XMMS 1.2.1 */
	g_module_symbol(instance->library, "player_quit",
			(gpointer) &instance->player_quit);
	/* Added in XMMS 1.2.6 */
	g_module_symbol(instance->library, "player_play_pause",
			(gpointer) &instance->player_play_pause);

	g_module_symbol(instance->library, "player_get_version",
			(gpointer *) &instance->player_get_version);
	g_module_symbol(instance->library, "player_playlist_add",
			(gpointer *) &instance->player_playlist_add);
	g_module_symbol(instance->library, "player_playlist_delete",
			(gpointer *) &instance->player_playlist_delete);
	g_module_symbol(instance->library, "player_main_win_toggle",
			(gpointer *) &instance->player_main_win_toggle);
	g_module_symbol(instance->library, "player_pl_win_toggle",
			(gpointer *) &instance->player_pl_win_toggle);
	g_module_symbol(instance->library, "player_eq_win_toggle",
			(gpointer *) &instance->player_eq_win_toggle);
	g_module_symbol(instance->library, "player_is_main_win",
			(gpointer *) &instance->player_is_main_win);
	g_module_symbol(instance->library, "player_is_pl_win",
			(gpointer *) &instance->player_is_pl_win);
	g_module_symbol(instance->library, "player_is_eq_win",
			(gpointer *) &instance->player_is_eq_win);
	g_module_symbol(instance->library, "player_playlist_add_url_string",
			(gpointer *) &instance->player_playlist_add_url_string);
	g_module_symbol(instance->library, "player_toggle_repeat",
			(gpointer *) &instance->player_toggle_repeat);
	g_module_symbol(instance->library, "player_toggle_shuffle",
			(gpointer *) &instance->player_toggle_shuffle);
	g_module_symbol(instance->library, "player_is_repeat",
			(gpointer *) &instance->player_is_repeat);
	g_module_symbol(instance->library, "player_is_shuffle",
			(gpointer *) &instance->player_is_shuffle);										
	/********************* end binding *********************/
	
	return instance;
}

