/*
 * Copyright (c) 2003 Patrick van Staveren <pvanstav@cs.wmich.edu>
 * Copyright (c) 2005 Mario Streiber <mario.streiber@gmx.de>
 * Copyright (c) 2005-2009 Kemal Ilgar Eroglu <ilgar_eroglu@yahoo.com> 
 * 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef PLAYERCTRL_H
#define PLAYERCTRL_H

#include <glib.h>


#define LIB_XMMS "xmms"
#define LIB_BEEP "beep"
#define LIB_AUDACIOUS "audclient"
#define LIB_AUDACIOUS_OLD "audacious"
#define LIB_AMAROK "amarok"
#define LIB_MPD "mpd"

#define CMD_XMMS "xmms"
#define CMD_BEEP "beep-media-player"
#define CMD_AUDACIOUS "audacious"
#define CMD_AUDACIOUS_OLD "audacious"
#define CMD_AMAROK "amarok"
#define CMD_MPD "mpd"



// define functions
typedef gboolean  (* _player_init) (gpointer cdata);
typedef gboolean  (* _player_deinit) (gpointer cdata);
typedef void (* _player_save_special_options) (gpointer cdata);
typedef void (* _player_read_special_options) (gpointer cdata);
typedef void (* _player_populate_special_options) (gpointer cdata);
typedef gboolean (* _player_update_special_options) (gpointer cdata);
typedef guint (* _player_get_menu_options) (gpointer cdata);

typedef void  (* _player_playlist) (gpointer cdata, gchar ** list, gint num, gboolean enqueue);
typedef void  (* _player_play) (gpointer cdata);
typedef void  (* _player_pause) (gpointer cdata);
typedef void  (* _player_stop) (gpointer cdata);
typedef gboolean  (* _player_is_playing) (gpointer cdata);
typedef gboolean  (* _player_is_paused) (gpointer cdata);
typedef gint  (* _player_get_playlist_pos) (gpointer cdata);
typedef void  (* _player_set_playlist_pos) (gpointer cdata, gint pos);
typedef gint  (* _player_get_playlist_length) (gpointer cdata);
typedef gint  (* _player_get_output_time) (gpointer cdata);
typedef void  (* _player_jump_to_time) (gpointer cdata, gint pos);
typedef void  (* _player_get_volume) (gpointer cdata, gint * vl, gint * vr);
typedef void  (* _player_set_volume) (gpointer cdata, gint vl, gint vr);
typedef gchar * (* _player_get_playlist_file) (gpointer cdata, gint pos);
typedef gchar * (* _player_get_playlist_title) (gpointer cdata, gint pos);
typedef gint  (* _player_get_playlist_time) (gpointer cdata, gint pos);
typedef void  (* _player_show_prefs_box) (gpointer cdata);
typedef void  (* _player_eject) (gpointer cdata);
typedef void  (* _player_playlist_prev) (gpointer cdata);
typedef void  (* _player_playlist_next) (gpointer cdata);
typedef gboolean  (* _player_is_running) (gpointer cdata);
typedef void  (* _player_quit) (gpointer cdata);
typedef void  (* _player_play_pause) (gpointer cdata);

typedef gchar *  (* _player_get_version) (gpointer cdata);
typedef void  (* _player_playlist_add) (gpointer cdata, GList * list);
typedef void  (* _player_playlist_delete) (gpointer cdata, gint pos);
typedef void  (* _player_main_win_toggle) (gpointer cdata, gboolean show);
typedef void  (* _player_pl_win_toggle) (gpointer cdata, gboolean show);
typedef void  (* _player_eq_win_toggle) (gpointer cdata, gboolean show);
typedef gboolean  (* _player_is_main_win) (gpointer cdata);
typedef gboolean  (* _player_is_pl_win) (gpointer cdata);
typedef gboolean  (* _player_is_eq_win) (gpointer cdata);
typedef void  (* _player_playlist_add_url_string) (gpointer cdata, gchar * string);
typedef void  (* _player_toggle_repeat) (gpointer cdata);
typedef void  (* _player_toggle_shuffle) (gpointer cdata);
typedef gboolean  (* _player_is_repeat) (gpointer cdata);
typedef gboolean  (* _player_is_shuffle) (gpointer cdata);
typedef void  (* _player_playlist_clear) (gpointer cdata);
typedef void  (* _player_show_jtf_box) (gpointer cdata);
typedef void  (* _player_toggle_aot) (gpointer cdata, gboolean ontop);

typedef struct {
	/** internally used. do not use directory */
	GModule *library;
	
	// functions
	_player_init player_init;
	_player_deinit player_deinit;
	_player_save_special_options player_save_special_options;	
	_player_read_special_options player_read_special_options;		
	_player_populate_special_options player_populate_special_options;
	_player_update_special_options player_update_special_options;	
	_player_get_menu_options player_get_menu_options;				
	_player_playlist player_playlist;
	_player_play player_play;
	_player_pause player_pause;
	_player_stop player_stop;
	_player_is_playing player_is_playing;
	_player_is_paused player_is_paused;
	_player_get_playlist_pos player_get_playlist_pos;
	_player_set_playlist_pos player_set_playlist_pos;
	_player_get_playlist_length player_get_playlist_length;
	_player_get_output_time player_get_output_time;
	_player_jump_to_time player_jump_to_time;
	_player_get_volume player_get_volume;
	_player_set_volume player_set_volume;
	_player_get_playlist_file player_get_playlist_file;
	_player_get_playlist_title player_get_playlist_title;
	_player_get_playlist_time player_get_playlist_time;
	_player_show_prefs_box player_show_prefs_box;
	_player_eject player_eject;
	_player_playlist_prev player_playlist_prev;
	_player_playlist_next player_playlist_next;
	_player_is_running player_is_running;
	_player_quit player_quit;
	_player_play_pause player_play_pause;

	_player_get_version player_get_version;
	_player_playlist_add player_playlist_add;
	_player_playlist_delete player_playlist_delete;	
	_player_main_win_toggle player_main_win_toggle;
	_player_pl_win_toggle player_pl_win_toggle;
	_player_eq_win_toggle player_eq_win_toggle;
	_player_is_main_win player_is_main_win;
	_player_is_pl_win player_is_pl_win;
	_player_is_eq_win player_is_eq_win;
	_player_playlist_add_url_string player_playlist_add_url_string;
	_player_toggle_repeat player_toggle_repeat;
	_player_toggle_shuffle player_toggle_shuffle;
	_player_is_repeat player_is_repeat;
	_player_is_shuffle player_is_shuffle;
	_player_playlist_clear player_playlist_clear;
	_player_show_jtf_box player_show_jtf_box;
	_player_toggle_aot player_toggle_aot;
} PlayerCtrl;

/**
 * used to player_control_load_library()
 */
typedef enum {
	PLAYER_NONE,
	PLAYER_XMMS,
	PLAYER_BEEP,
	PLAYER_AUDACIOUS,
	PLAYER_AUDACIOUS_OLD,
	PLAYER_AMAROK,
	PLAYER_MPD
} Players;

#define PLAYER_LAST PLAYER_MPD

/**
 * player control functions
 * filled by zero if a player not support specific function
 * @see xmmsctrl.h or beepctrl.h
 */

/**
 * get command name from ID
 */
gchar * player_control_get_command(Players id);

/**
 * load library from ID
 * @return returns NULL if library is not found
 */
PlayerCtrl * player_control_get_instance(Players id);

/**
 * unload library
 * must call this function after using PlayerCtrl instance
 * @see player_control_get_instance(), player_control_load_library()
 */
void player_control_close(PlayerCtrl *instance);

/**
 * check whether a library is exist or not
 */
gboolean player_control_has_library(gchar *libname);

/**
 * load library
 * @param lib_name player library name (Eg. "xmms", "beep", "audacious")
 * @return returns NULL if library is not found
 */
PlayerCtrl * player_control_load_library(gchar *libname);

#endif
